//	----------------------------------------------------------------------------
//	MemoryAllocator.cp
//	----------------------------------------------------------------------------
//	By Jean-Franois Brouillet, Copyright  1995
//
//	V0.001		951023	JFB		Use of dynarray class to ease error recovery
//								when exceptions are thrown.
//	V0.000		951017	JFB		Creation
//
//	No restriction on use. But please, use with caution as this is not
//	thoroughly tested. Also, understand when the use of such an allocator is
//	appropriate for your program : speed doesn't come for free !
//
//	Principle:
//	---------
//	Let's assume we only manage one kind of objects, all of them the same size.
//	Our strategy is as follows :
//		a)	we keep a "global" variable which is to be the head of any freed
//			object.
//		b)	when we're requested for an object storage, we first check our head.
//			if the head is 0 then we have no choice but draw the needed bytes
//			from whatever "system allocator" we can rely on.
//		c)	when we're asked to free an object, instead of returning it to the
//			system, we cache it as follows:
//				a)	since this object is to be freed (its destructor has *already*
//					been called) we consider it raw storage and then can map
//					whatever structure of our liking over it.
//				b)	we postulate that this object is of type 'heapNode' and
//					make its 'next' field point to our head.
//				c)	we set our head to point to this freed object which, in fact
//					will be recycled next time new is called.
//
//	The remaining details are that we keep the size of the object within the object
//	itself for later retreival, and that we don't have a unique head, but a head
//	for each different size of objects allocated. These (size,head) pairs are stored
//	into two parallel arrays (m_ObjectSizeArray, m_ObjectArray) which are dynamically
//	grown as the need arises. Of course, for the storage of these arrays we can't rely
//	on our scheme (which is basically for dealing with fixed size datums, as C++
//	objects usually are).
//
//	Benefits:
//	--------
//	This scheme is auto adaptative in that *no* modification of client code is
//	required (except for an appropriate operator new/delete pair).
//	Measured speed improvments range from 5 to 10 times faster than usual new/delete
//	(Your mileage may vary)
//	When storage is gained from the cache, the algorithm performs in constant time
//	for allocation (it always performs in constant time for deletion)
//
//	Drawbacks:
//	---------
//	As freed objects are not really freed, there's no way a request for 2n bytes
//	could be fullfilled with two 'adjacent' n bytes objects = they are not on the
//	same list! For this purose a 'releaseAll' method is provided, which 'purges'
//	the various free lists, and helds back the objects to the system allocator.
//	This routine must be used with caution, as it defeats the very purpose of this
//	allocator.

#include	<exception>
#include	<new>
#include	<string.h>
#include	"MemoryAllocator.h"
#include	"Accessors.t"
#include	"dynarray.t"

MemoryAllocator *
MemoryAllocator::M_Singleton = 0;

//	----------------------------------------------------------------------------
//	MemoryAllocator::GetAllocator
//	----------------------------------------------------------------------------
MemoryAllocator *
MemoryAllocator::GetAllocator(void)
{
	if (M_Singleton == 0)
		M_Singleton = new MemoryAllocator;

	return M_Singleton;
}

//	----------------------------------------------------------------------------
//	MemoryAllocator::MemoryAllocator
//	----------------------------------------------------------------------------
MemoryAllocator::MemoryAllocator()
{
	if (M_Singleton == 0)
		M_Singleton = this;
	else
		throw	xalloc();
}

//	----------------------------------------------------------------------------
//	MemoryAllocator::~MemoryAllocator
//	----------------------------------------------------------------------------
MemoryAllocator::~MemoryAllocator()
{
	releaseAll();
	M_Singleton = 0;
}

//	----------------------------------------------------------------------------
//	MemoryAllocator::releaseAll
//	----------------------------------------------------------------------------
void
MemoryAllocator::releaseAll(
	void)
{
	if (m_ObjectArray.count())
	{
		for (size_t vIndex = 0; vIndex < m_ObjectArray.count(); ++vIndex)
		{
			void * vObject = m_ObjectArray[vIndex];
			while(vObject)
			{
				if (((MemoryAllocator::heapNode *) vObject)->size
					== m_ObjectSizeArray[vIndex])
				{
					void *	vNext = ((MemoryAllocator::heapNode *) vObject)->next;
					::operator delete(vObject);
					vObject = vNext;
				}
				else
				{
					throw xalloc("MemoryAllocator::releaseAll found corrupted heap!");
					vObject = 0;
				}
			}	
		}
	}
}

//	----------------------------------------------------------------------------
//	MemoryAllocator::allocate
//	----------------------------------------------------------------------------
void *
MemoryAllocator::allocate(
	size_t inBytes)
{
size_t			vRealSize = sizeof(size_t) + ::align32bits(inBytes);
MemoryAllocator::heapNode * &
				vHeapHeader = getHeapHeader(vRealSize);
void *			vObject;

	if (vHeapHeader)
	{
		vObject = (void *) vHeapHeader;
		vHeapHeader = vHeapHeader->next;
	}
	else
		vObject = ::operator new(vRealSize);

	::stash(vRealSize, &vObject);	//	done even if vHeapHeader != 0 !!!

	return vObject;
}

//	----------------------------------------------------------------------------
//	MemoryAllocator::release
//	----------------------------------------------------------------------------
void
MemoryAllocator::release(
	void * inObject)
{
size_t			vRealSize;

	::prefetch(vRealSize, &inObject);
	MemoryAllocator::heapNode * &	vHeapHeader = getHeapHeader(vRealSize);
	((MemoryAllocator::heapNode *) inObject)->next = vHeapHeader;
	vHeapHeader = (MemoryAllocator::heapNode *) inObject;
}

//	----------------------------------------------------------------------------
//	MemoryAllocator::getHeapHeader
//	----------------------------------------------------------------------------
MemoryAllocator::heapNode * &
MemoryAllocator::getHeapHeader(
	size_t inObjectSize)
{
size_t		vIndex;

	for (vIndex = 0; vIndex < m_ObjectSizeArray.count(); ++vIndex)
	{
		if (m_ObjectSizeArray[vIndex] == inObjectSize)
			return m_ObjectArray[vIndex];
	}

	size_t	vLastPosition = m_ObjectSizeArray.count();

	m_ObjectSizeArray.resize(1 + vLastPosition);	//	may fail
	m_ObjectSizeArray[vLastPosition] = 0;			//	keep the list consistent
	m_ObjectArray.resize(1 + vLastPosition);		//	may fail
	m_ObjectArray[vLastPosition] = 0;				//	we've no linked object yet
	m_ObjectSizeArray[vLastPosition] = inObjectSize;//	store actual size *now*

	return m_ObjectArray[vLastPosition];
}
